@@ -4,15 +4,16 @@ from client import Client
 
 class EtcdResult(collections.namedtuple(
     'EtcdResult',
-    [
-        'action',
-        'index',
-        'key',
-        'prevValue',
-        'value',
-        'expiration',
-        'ttl',
-        'newKey'])):
+        ['action',
+         'index',
+         'key',
+         'prevValue',
+         'value',
+         'expiration',
+         'ttl',
+         'newKey',
+         'dir',
+         ])):
 
     def __new__(
             cls,
@@ -23,7 +24,9 @@ class EtcdResult(collections.namedtuple(
             value=None,
             expiration=None,
             ttl=None,
-            newKey=None):
+            newKey=None,
+            dir=False
+    ):
         return super(EtcdResult, cls).__new__(
             cls,
             action,
@@ -33,7 +36,9 @@ class EtcdResult(collections.namedtuple(
             value,
             expiration,
             ttl,
-            newKey)
+            newKey,
+            dir
+        )
 
 
 class EtcdException(Exception):
@@ -42,3 +47,11 @@ class EtcdException(Exception):
     """
 
     pass
+
+# Attempt to enable urllib3's SNI support, if possible
+# Blatantly copied from requests.
+try:
+    from urllib3.contrib import pyopenssl
+    pyopenssl.inject_into_urllib3()
+except ImportError:
+    pass
