@@ -8,6 +8,7 @@
 """
 import urllib3
 import json
+import ssl
 
 import etcd
 
@@ -16,18 +17,25 @@ class Client(object):
     """
     Client for etcd, the distributed log service using raft.
     """
+
     def __init__(
             self,
             host='127.0.0.1',
             port=4001,
             read_timeout=60,
             allow_redirect=True,
-            protocol='http'):
+            protocol='http',
+            cert=None,
+            ca_cert=None,
+            allow_reconnect=False,
+    ):
         """
         Initialize the client.
 
         Args:
-            host (str):  IP to connect to.
+            host (mixed):
+                           If a string, IP to connect to.
+                           If a tuple ((host, port), (host, port), ...)
 
             port (int):  Port used to connect to etcd.
 
@@ -37,15 +45,39 @@ class Client(object):
 
             protocol (str):  Protocol used to connect to etcd.
 
+            cert (mixed):   If a string, the whole ssl client certificate;
+                            if a tuple, the cert and key file names.
+
+            ca_cert (str): The ca certificate. If pressent it will enable
+                           validation.
+
+            allow_reconnect (bool): allow the client to reconnect to another
+                                    etcd server in the cluster in the case the
+                                    default one does not respond.
+
         """
-        self._host = host
-        self._port = port
+        self._machines_cache = []
+
         self._protocol = protocol
-        self._base_uri = "%s://%s:%d" % (protocol, host, port)
+
+        def uri(protocol, host, port):
+            return '%s://%s:%d' % (protocol, host, port)
+
+        if not isinstance(host, tuple):
+            self._host = host
+            self._port = port
+        else:
+            self._host, self._port = host[0]
+            self._machines_cache.extend(
+                [uri(self._protocol, *conn) for conn in host])
+
+        self._base_uri = uri(self._protocol, self._host, self._port)
+
         self.version_prefix = '/v1'
 
         self._read_timeout = read_timeout
         self._allow_redirect = allow_redirect
+        self._allow_reconnect = allow_reconnect
 
         self._MGET = 'GET'
         self._MPOST = 'POST'
@@ -70,7 +102,37 @@ class Client(object):
             500: etcd.EtcdException,
             999: etcd.EtcdException}
 
-        self.http = urllib3.PoolManager(10)
+        # SSL Client certificate support
+
+        kw = {}
+
+        if protocol == 'https':
+            # If we don't allow TLSv1, clients using older version of OpenSSL
+            # (<1.0) won't be able to connect.
+            kw['ssl_version'] = ssl.PROTOCOL_TLSv1
+
+            if cert:
+                if isinstance(cert, tuple):
+                    # Key and cert are separate
+                    kw['cert_file'] = cert[0]
+                    kw['key_file'] = cert[1]
+                else:
+                    #combined certificate
+                    kw['cert_file'] = cert
+
+            if ca_cert:
+                kw['ca_certs'] = ca_cert
+                kw['cert_reqs'] = ssl.CERT_REQUIRED
+
+        self.http = urllib3.PoolManager(num_pools=10, **kw)
+
+        if self._allow_reconnect:
+            # we need the set of servers in the cluster in order to try
+            # reconnecting upon error.
+            self._machines_cache = self.machines
+            self._machines_cache.remove(self._base_uri)
+        else:
+            self._machines_cache = []
 
     @property
     def base_uri(self):
@@ -271,7 +333,8 @@ class Client(object):
             key (str):  Key.
 
         Returns:
-            client.EtcdResult (or an array of client.EtcdResult if a subtree is queried)
+            client.EtcdResult (or an array of client.EtcdResult if a
+            subtree is queried)
 
         Raises:
             KeyValue:  If the key doesn't exists.
@@ -326,25 +389,49 @@ class Client(object):
         except:
             raise etcd.EtcdException('Unable to decode server response')
 
+    def _next_server(self):
+        """ Selects the next server in the list, refreshes the server list. """
+        try:
+            return self._machines_cache.pop()
+        except IndexError:
+            raise etcd.EtcdException('No more machines in the cluster')
+
     def api_execute(self, path, method, params=None):
         """ Executes the query. """
-        if (method == self._MGET) or (method == self._MDELETE):
-            response = self.http.request(
-                method,
-                self._base_uri + path,
-                fields=params,
-                redirect=self.allow_redirect)
 
-        elif method == self._MPOST:
-            response = self.http.request_encode_body(
-                method,
-                self._base_uri+path,
-                fields=params,
-                encode_multipart=False,
-                redirect=self.allow_redirect)
+        some_request_failed = False
+        response = False
+
+        while not response:
+            try:
+                url = self._base_uri + path
+
+                if (method == self._MGET) or (method == self._MDELETE):
+                    response = self.http.request(
+                        method,
+                        url,
+                        fields=params,
+                        redirect=self.allow_redirect)
+
+                elif method == self._MPOST:
+                    response = self.http.request_encode_body(
+                        method,
+                        url,
+                        fields=params,
+                        encode_multipart=False,
+                        redirect=self.allow_redirect)
+
+            except urllib3.exceptions.MaxRetryError:
+                self._base_uri = self._next_server()
+                some_request_failed = True
+
+        if some_request_failed:
+            self._machines_cache = self.machines
+            self._machines_cache.remove(self._base_uri)
 
         if response.status == 200:
             return response.data
+
         else:
             try:
                 error = json.loads(response.data)
