@@ -138,7 +138,7 @@ class TestClientRequest(unittest.TestCase):
         """Can get multiple values"""
         client = etcd.Client()
         client.api_execute = mock.Mock(
-            return_value =
+            return_value=
             '[{"action":"GET",'
             '"key":"/testdir/key1",'
             '"value":"test1",'
@@ -151,15 +151,43 @@ class TestClientRequest(unittest.TestCase):
         result = client.get('/testdir')
         self.assertEquals([
             etcd.EtcdResult(
-            **{u'action': u'GET',
-               u'index': 190,
-               u'key': u'/testdir/key1',
-               u'value': u'test1'}),
+                **{u'action': u'GET',
+                   u'index': 190,
+                   u'key': u'/testdir/key1',
+                   u'value': u'test1'}),
             etcd.EtcdResult(
-            **{u'action': u'GET',
-               u'index': 190,
-               u'key': u'/testdir/key2',
-               u'value': u'test2'})
+                **{u'action': u'GET',
+                   u'index': 190,
+                   u'key': u'/testdir/key2',
+                   u'value': u'test2'})
+        ], result)
+
+    def test_get_subdirs(self):
+        """ Can understand dirs in results """
+        client = etcd.Client()
+        client.api_execute = mock.Mock(
+            return_value=
+            '[{"action":"GET",'
+            '"key":"/testdir/key1",'
+            '"value":"test1",'
+            '"index":190},'
+            '{"action":"GET",'
+            '"key":"/testdir/key2",'
+            '"dir": true,'
+            '"index":191}]'
+        )
+        result = client.get('/testdir')
+        self.assertEquals([
+            etcd.EtcdResult(
+                **{u'action': u'GET',
+                   u'index': 190,
+                   u'key': u'/testdir/key1',
+                   u'value': u'test1'}),
+            etcd.EtcdResult(
+                **{u'action': u'GET',
+                   u'index': 191,
+                   u'key': u'/testdir/key2',
+                   u'dir': True})
         ], result)
 
     def test_not_in(self):
